@@ -12,14 +12,13 @@ __metaclass__ = type
 import traceback
 import re
 import json
-
 from itertools import chain
 
 from ansible.module_utils.common.text.converters import to_text
 
 from ansible.module_utils._text import to_native
 from ansible.module_utils.common.collections import is_iterable
-from ansible.module_utils.basic import AnsibleModule, missing_required_lib
+from ansible.module_utils.basic import AnsibleModule, missing_required_lib, _load_params
 from ansible.module_utils.urls import open_url
 
 PYNETBOX_IMP_ERR = None
@@ -34,7 +33,13 @@ except ImportError:
 
 # Used to map endpoints to applications dynamically
 API_APPS_ENDPOINTS = dict(
-    circuits=["circuits", "circuit_types", "circuit_terminations", "providers"],
+    circuits=[
+        "circuits",
+        "circuit_types",
+        "circuit_terminations",
+        "providers",
+        "provider_networks",
+    ],
     dcim=[
         "cables",
         "console_ports",
@@ -70,7 +75,7 @@ API_APPS_ENDPOINTS = dict(
         "site_groups",
         "virtual_chassis",
     ],
-    extras=["tags"],
+    extras=["config_contexts", "tags"],
     ipam=[
         "aggregates",
         "ip_addresses",
@@ -96,6 +101,7 @@ QUERY_TYPES = dict(
     cluster="name",
     cluster_group="slug",
     cluster_type="slug",
+    config_context="name",
     device="name",
     device_role="slug",
     device_type="slug",
@@ -107,7 +113,9 @@ QUERY_TYPES = dict(
     manufacturer="slug",
     nat_inside="address",
     nat_outside="address",
+    parent_location="slug",
     parent_region="slug",
+    parent_site_group="slug",
     parent_tenant_group="slug",
     power_panel="name",
     power_port="name",
@@ -118,6 +126,7 @@ QUERY_TYPES = dict(
     primary_ip4="address",
     primary_ip6="address",
     provider="slug",
+    provider_network="name",
     rack="name",
     rack_group="slug",
     rack_role="slug",
@@ -150,7 +159,9 @@ CONVERT_TO_ID = {
     "circuits.circuittermination": "circuit_terminations",
     "cluster": "clusters",
     "cluster_group": "cluster_groups",
+    "cluster_groups": "cluster_groups",
     "cluster_type": "cluster_types",
+    "config_context": "config_contexts",
     "dcim.consoleport": "console_ports",
     "dcim.consoleserverport": "console_server_ports",
     "dcim.frontport": "front_ports",
@@ -162,6 +173,7 @@ CONVERT_TO_ID = {
     "device": "devices",
     "device_role": "device_roles",
     "device_type": "device_types",
+    "device_types": "device_types",
     "export_targets": "route_targets",
     "group": "tenant_groups",
     "import_targets": "route_targets",
@@ -177,8 +189,13 @@ CONVERT_TO_ID = {
     "nat_inside": "ip_addresses",
     "nat_outside": "ip_addresses",
     "platform": "platforms",
+    "parent_location": "locations",
+    "parent_interface": "interfaces",
+    "parent_vm_interface": "interfaces",
     "parent_region": "regions",
+    "parent_site_group": "site_groups",
     "parent_tenant_group": "tenant_groups",
+    "platforms": "platforms",
     "power_panel": "power_panels",
     "power_port": "power_ports",
     "power_port_template": "power_port_templates",
@@ -187,23 +204,29 @@ CONVERT_TO_ID = {
     "primary_ip4": "ip_addresses",
     "primary_ip6": "ip_addresses",
     "provider": "providers",
+    "provider_network": "provider_networks",
     "rack": "racks",
     "rack_group": "rack_groups",
     "rack_role": "rack_roles",
     "region": "regions",
+    "regions": "regions",
     "rear_port": "rear_ports",
     "rear_port_template": "rear_port_templates",
     "rir": "rirs",
+    "roles": "device_roles",
     "route_targets": "route_targets",
     # Just a placeholder as scope can be several different types including sites.
     "scope": "sites",
     "services": "services",
     "site": "sites",
     "site_group": "site_groups",
+    "sites": "sites",
     "tags": "tags",
     "tagged_vlans": "vlans",
     "tenant": "tenants",
+    "tenants": "tenants",
     "tenant_group": "tenant_groups",
+    "tenant_groups": "tenant_groups",
     "termination_a": "interfaces",
     "termination_b": "interfaces",
     "untagged_vlan": "vlans",
@@ -225,6 +248,7 @@ ENDPOINT_NAME_MAPPING = {
     "clusters": "cluster",
     "cluster_groups": "cluster_group",
     "cluster_types": "cluster_type",
+    "config_contexts": "config_context",
     "console_ports": "console_port",
     "console_port_templates": "console_port_template",
     "console_server_ports": "console_server_port",
@@ -251,6 +275,7 @@ ENDPOINT_NAME_MAPPING = {
     "power_port_templates": "power_port_template",
     "prefixes": "prefix",
     "providers": "provider",
+    "provider_networks": "provider_network",
     "racks": "rack",
     "rack_groups": "rack_group",
     "rack_roles": "rack_role",
@@ -283,6 +308,21 @@ ALLOWED_QUERY_PARAMS = {
     "cluster": set(["name", "type"]),
     "cluster_group": set(["slug"]),
     "cluster_type": set(["slug"]),
+    "config_context": set(
+        [
+            "name",
+            "regions",
+            "sites",
+            "roles",
+            "device_types",
+            "platforms",
+            "cluster_groups",
+            "clusters",
+            "tenant_groups",
+            "tenants",
+            "tags",
+        ]
+    ),
     "console_port": set(["name", "device"]),
     "console_port_template": set(["name", "device_type"]),
     "console_server_port": set(["name", "device"]),
@@ -314,7 +354,11 @@ ALLOWED_QUERY_PARAMS = {
     "manufacturer": set(["slug"]),
     "master": set(["name"]),
     "nat_inside": set(["vrf", "address"]),
+    "parent_location": set(["slug"]),
+    "parent_interface": set(["name"]),
+    "parent_vm_interface": set(["name"]),
     "parent_region": set(["slug"]),
+    "parent_site_group": set(["slug"]),
     "parent_tenant_group": set(["slug"]),
     "platform": set(["slug"]),
     "power_feed": set(["name", "power_panel"]),
@@ -327,6 +371,7 @@ ALLOWED_QUERY_PARAMS = {
     "primary_ip4": set(["address", "vrf"]),
     "primary_ip6": set(["address", "vrf"]),
     "provider": set(["slug"]),
+    "provider_network": set(["name"]),
     "rack": set(["name", "site"]),
     "rack_group": set(["slug"]),
     "rack_role": set(["slug"]),
@@ -338,6 +383,7 @@ ALLOWED_QUERY_PARAMS = {
     "route_target": set(["name"]),
     "services": set(["device", "virtual_machine", "name", "port", "protocol"]),
     "site": set(["slug"]),
+    "site_group": set(["slug"]),
     "tags": set(["slug"]),
     "tagged_vlans": set(["group", "name", "site", "vid", "vlan_group", "tenant"]),
     "tenant": set(["slug"]),
@@ -399,14 +445,18 @@ REQUIRED_ID_FIND = {
     "vlans": set(["status"]),
 }
 
-# This is used to map non-clashing keys to Netbox API compliant keys to prevent bad logic in code for similar keys but different modules
+# This is used to map non-clashing keys to NetBox API compliant keys to prevent bad logic in code for similar keys but different modules
 CONVERT_KEYS = {
     "assigned_object": "assigned_object_id",
     "scope": "scope_id",
     "circuit_type": "type",
     "cluster_type": "type",
     "cluster_group": "group",
+    "parent_location": "parent",
+    "parent_interface": "parent",
+    "parent_vm_interface": "parent",
     "parent_region": "parent",
+    "parent_site_group": "parent",
     "parent_tenant_group": "parent",
     "power_port_template": "power_port",
     "prefix_role": "role",
@@ -414,6 +464,7 @@ CONVERT_KEYS = {
     "rack_role": "role",
     "rear_port_template": "rear_port",
     "rear_port_template_position": "rear_port_position",
+    "site_group": "group",
     "tenant_group": "group",
     "termination_a": "termination_a_id",
     "termination_b": "termination_b_id",
@@ -463,12 +514,13 @@ NETBOX_ARG_SPEC = dict(
     state=dict(required=False, default="present", choices=["present", "absent"]),
     query_params=dict(required=False, type="list", elements="str"),
     validate_certs=dict(type="raw", default=True),
+    cert=dict(type="raw", required=False),
 )
 
 
 class NetboxModule(object):
     """
-    Initialize connection to Netbox, sets AnsibleModule passed in to
+    Initialize connection to NetBox, sets AnsibleModule passed in to
     self.module to be used throughout the class
     :params module (obj): Ansible Module object
     :params endpoint (str): Used to tell class which endpoint the logic needs to follow
@@ -486,18 +538,20 @@ class NetboxModule(object):
             self.module.fail_json(
                 msg=missing_required_lib("pynetbox"), exception=PYNETBOX_IMP_ERR
             )
-        # These should not be required after making connection to Netbox
+        # These should not be required after making connection to NetBox
         url = self.module.params["netbox_url"]
         token = self.module.params["netbox_token"]
         ssl_verify = self.module.params["validate_certs"]
+        cert = self.module.params["cert"]
 
-        # Attempt to initiate connection to Netbox
+        # Attempt to initiate connection to NetBox
         if nb_client is None:
-            self.nb = self._connect_netbox_api(url, token, ssl_verify)
+            self.nb = self._connect_netbox_api(url, token, ssl_verify, cert)
         else:
             self.nb = nb_client
             try:
                 self.version = self.nb.version
+                self.full_version = self.nb.status().get("netbox-version")
             except AttributeError:
                 self.module.fail_json(msg="Must have pynetbox >=4.1.0")
 
@@ -536,23 +590,26 @@ class NetboxModule(object):
         elif g_major == l_major and g_minor > l_minor:
             return True
 
-    def _connect_netbox_api(self, url, token, ssl_verify):
+    def _connect_netbox_api(self, url, token, ssl_verify, cert):
         try:
             session = requests.Session()
             session.verify = ssl_verify
+            if cert:
+                session.cert = tuple(i for i in cert)
             nb = pynetbox.api(url, token=token)
             nb.http_session = session
             try:
                 self.version = nb.version
+                self.full_version = nb.status().get("netbox-version")
             except AttributeError:
                 self.module.fail_json(msg="Must have pynetbox >=4.1.0")
             except Exception:
                 self.module.fail_json(
-                    msg="Failed to establish connection to Netbox API"
+                    msg="Failed to establish connection to NetBox API"
                 )
             return nb
         except Exception:
-            self.module.fail_json(msg="Failed to establish connection to Netbox API")
+            self.module.fail_json(msg="Failed to establish connection to NetBox API")
 
     def _nb_endpoint_get(self, nb_endpoint, query_params, search_item):
         try:
@@ -622,7 +679,7 @@ class NetboxModule(object):
         """
         Used to change non-clashing keys for each module into identical keys that are required
         to be passed to pynetbox
-        ex. rack_role back into role to pass to Netbox
+        ex. rack_role back into role to pass to NetBox
         Returns data
         :params data (dict): Data dictionary after _find_ids method ran
         """
@@ -685,10 +742,10 @@ class NetboxModule(object):
     ):
         """
         :returns dict(query_dict): Returns a query dictionary built using mappings to dynamically
-        build available query params for Netbox endpoints
+        build available query params for NetBox endpoints
         :params parent(str): This is either a key from `_find_ids` or a string passed in to determine
         which keys in the data that we need to use to construct `query_dict`
-        :params module_data(dict): Uses the data provided to the Netbox module
+        :params module_data(dict): Uses the data provided to the NetBox module
         :params child(dict): This is used within `_find_ids` and passes the inner dictionary
         to build the appropriate `query_dict` for the parent
         """
@@ -729,6 +786,23 @@ class NetboxModule(object):
             # This is to skip any potential changes using module_data when the user
             # provides user_query_params
             pass
+
+        elif parent == "prefix" and module_data.get("parent"):
+            query_dict.update({"prefix": module_data["parent"]})
+
+        elif parent == "parent_interface" and module_data.get("device"):
+            if not child:
+                query_dict["name"] = module_data.get("parent_interface")
+
+            if isinstance(module_data.get("device"), int):
+                query_dict.update({"device_id": module_data.get("device")})
+            else:
+                query_dict.update({"device": module_data.get("device")})
+
+        elif parent == "parent_vm_interface" and module_data.get("virtual_machine"):
+            if not child:
+                query_dict["name"] = module_data["parent_vm_interface"]
+
         elif parent == "lag":
             if not child:
                 query_dict["name"] = module_data["lag"]
@@ -741,9 +815,6 @@ class NetboxModule(object):
             else:
                 query_dict.update({"device": module_data["device"]})
 
-        elif parent == "prefix" and module_data.get("parent"):
-            query_dict.update({"prefix": module_data["parent"]})
-
         elif parent == "ip_addresses":
             if isinstance(module_data["device"], int):
                 query_dict.update({"device_id": module_data["device"]})
@@ -765,7 +836,7 @@ class NetboxModule(object):
                 )
 
         elif parent == "virtual_chassis":
-            query_dict = {"q": self.module.params["data"].get("master")}
+            query_dict.update({"master": self.module.params["data"].get("master")})
 
         elif parent == "rear_port" and self.endpoint == "front_ports":
             if isinstance(module_data.get("rear_port"), str):
@@ -829,7 +900,7 @@ class NetboxModule(object):
                 msg="Failed to fetch endpoint choices to validate against. This requires a write-enabled token. Make sure the token is write-enabled. If looking to fetch only information, use either the inventory or lookup plugin."
             )
 
-        choices = [x for x in chain.from_iterable(endpoint_choices.values())]
+        choices = list(chain.from_iterable(endpoint_choices.values()))
 
         for item in choices:
             if item["display_name"].lower() == search.lower():
@@ -881,7 +952,7 @@ class NetboxModule(object):
                         self.version, "2.9", greater_or_equal=True
                     )
                     and k == "tags"
-                ):
+                ) or (self.endpoint == "config_contexts" and k == "tags"):
                     continue
                 if k == "termination_a":
                     endpoint = CONVERT_TO_ID[data.get("termination_a_type")]
@@ -910,7 +981,18 @@ class NetboxModule(object):
                 elif isinstance(v, list):
                     id_list = list()
                     for list_item in v:
-                        if k == "tags" and isinstance(list_item, str):
+                        if k in (
+                            "regions",
+                            "sites",
+                            "roles",
+                            "device_types",
+                            "platforms",
+                            "cluster_groups",
+                            "clusters",
+                            "tenant_groups",
+                            "tenants",
+                            "tags",
+                        ) and isinstance(list_item, str):
                             temp_dict = {"slug": self._to_slug(list_item)}
                         elif isinstance(list_item, dict):
                             norm_data = self._normalize_data(list_item)
@@ -933,6 +1015,7 @@ class NetboxModule(object):
                 else:
                     if k in [
                         "lag",
+                        "parent_interface",
                         "rear_port",
                         "rear_port_template",
                         "power_port",
@@ -947,6 +1030,12 @@ class NetboxModule(object):
                                 ENDPOINT_NAME_MAPPING[endpoint], "q"
                             ): search
                         }
+                    elif k == "parent_vm_interface":
+                        nb_app = getattr(self.nb, "virtualization")
+                        nb_endpoint = getattr(nb_app, endpoint)
+                        query_params = self._build_query_params(
+                            k, data, user_query_params
+                        )
                     else:
                         query_params = {QUERY_TYPES.get(k, "q"): search}
                     query_id = self._nb_endpoint_get(nb_endpoint, query_params, k)
@@ -978,10 +1067,10 @@ class NetboxModule(object):
 
     def _normalize_data(self, data):
         """
-        :returns data (dict): Normalized module data to formats accepted by Netbox searches
+        :returns data (dict): Normalized module data to formats accepted by NetBox searches
         such as changing from user specified value to slug
         ex. Test Rack -> test-rack
-        :params data (dict): Original data from Netbox module
+        :params data (dict): Original data from NetBox module
         """
         for k, v in data.items():
             if isinstance(v, dict):
@@ -1025,9 +1114,9 @@ class NetboxModule(object):
         return data
 
     def _create_netbox_object(self, nb_endpoint, data):
-        """Create a Netbox object.
+        """Create a NetBox object.
         :returns tuple(serialized_nb_obj, diff): tuple of the serialized created
-        Netbox object and the Ansible diff.
+        NetBox object and the Ansible diff.
         """
         if self.check_mode:
             nb_obj = data
@@ -1041,7 +1130,7 @@ class NetboxModule(object):
         return nb_obj, diff
 
     def _delete_netbox_object(self):
-        """Delete a Netbox object.
+        """Delete a NetBox object.
         :returns diff (dict): Ansible diff
         """
         if not self.check_mode:
@@ -1054,9 +1143,9 @@ class NetboxModule(object):
         return diff
 
     def _update_netbox_object(self, data):
-        """Update a Netbox object.
+        """Update a NetBox object.
         :returns tuple(serialized_nb_obj, diff): tuple of the serialized updated
-        Netbox object and the Ansible diff.
+        NetBox object and the Ansible diff.
         """
         serialized_nb_obj = self.nb_object.serialize()
         updated_obj = serialized_nb_obj.copy()
@@ -1065,6 +1154,24 @@ class NetboxModule(object):
             serialized_nb_obj["tags"] = set(serialized_nb_obj["tags"])
             updated_obj["tags"] = set(data["tags"])
 
+        # Ensure idempotency for site and virtual machine on version pre-3.0
+        version_pre_30 = self._version_check_greater("3.0", self.version)
+        if (
+            serialized_nb_obj.get("latitude")
+            and data.get("latitude")
+            and version_pre_30
+        ):
+            updated_obj["latitude"] = str(data["latitude"])
+        if (
+            serialized_nb_obj.get("longitude")
+            and data.get("longitude")
+            and version_pre_30
+        ):
+            updated_obj["longitude"] = str(data["longitude"])
+
+        if serialized_nb_obj.get("vcpus") and data.get("vcpus") and version_pre_30:
+            updated_obj["vcpus"] = "{0:.2f}".format(data["vcpus"])
+
         if serialized_nb_obj == updated_obj:
             return serialized_nb_obj, None
         else:
@@ -1143,11 +1250,11 @@ class NetboxAnsibleModule(AnsibleModule):
     """
     Creating this due to needing to override some functionality to provide required_together, required_if
     and will be able to override more in the future.
-    This is due to the Netbox modules having the module arguments within a key in the argument spec, using suboptions rather than
+    This is due to the NetBox modules having the module arguments within a key in the argument spec, using suboptions rather than
     having all module arguments within the regular argument spec.
 
-    Didn't want to change that functionality of the Netbox modules as its disruptive and we're required to send a specific payload
-    to the Netbox API
+    Didn't want to change that functionality of the NetBox modules as its disruptive and we're required to send a specific payload
+    to the NetBox API
     """
 
     def __init__(
@@ -1163,18 +1270,37 @@ class NetboxAnsibleModule(AnsibleModule):
         required_if=None,
         required_by=None,
     ):
+        # Sets each check to None so they are not run in AnsibleModule
         super().__init__(
             argument_spec,
             bypass_checks=False,
             no_log=False,
-            mutually_exclusive=mutually_exclusive,
-            required_together=required_together,
-            required_one_of=required_one_of,
+            mutually_exclusive=None,
+            required_together=None,
+            required_one_of=None,
             add_file_common_args=False,
             supports_check_mode=supports_check_mode,
-            required_if=required_if,
+            required_if=None,
         )
 
+        # Quick fix to support ansible-core 2.11
+        #
+        # Load the params manually as the self.params already have the defaults set
+        params = _load_params()
+
+        # Run each check manually providing the params
+        if mutually_exclusive:
+            self._check_mutually_exclusive(mutually_exclusive, param=params)
+
+        if required_together:
+            self._check_required_together(required_together, param=params)
+
+        if required_one_of:
+            self._check_required_one_of(required_one_of, param=params)
+
+        if required_if:
+            self._check_required_if(required_if, param=params)
+
     def _check_mutually_exclusive(self, spec, param=None):
         if param is None:
             param = self.params
